// This software was written by Gregory L Montgomery
// and is completely covered by the 
// GPL General Public License

// Date of release 1/7/1998 

/*  Greg Montgomery */
/*  main.c final project software */


/*  File:  main.c      */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include "defs.h"
#include "mux.h"
#include "ad.h"
#include "ADC0804.h"
#include "Gen.h"

/*********  Global variables **************************************/

Display *     Gdisplay;             /* global display */
unsigned long Gforeground;          /* window foreground color */
unsigned long Gbackground;          /* window background color */
unsigned long Gsquare[NUM_BUTTONS]; /* color of colored square on chess board */
unsigned long Glinecolor;           /* color of line y=x on chess board */
int           Gevent_mask;          /* event mask for window */
Window        Gwindow;	            /* global window */
GC            Ggc;		    /* context for drawing */
int           Gscreen;		    /* global screen */
int           Gwidth;		    /* width of global window */
int           Gheight;		    /* height of global window */
unsigned int  Gdepth;	            /* depth of global window */
int           Gdim = 2;		    /* dimension of chess board */
int           color[MAX_COLORS];  /* colors are stored in this array */
static int    XSize, YSize;
BOX_REC Button[NUM_BUTTONS];
mADC0804 AD;
Generic  Temp;

/********* Function prototypes ************************************/

void initXwindows (int argc, char *argv[]);
void doEventLoop();
void endXwindows (void);
void handleExpose (void);
void draw_menu();
void Init_Menu_Struct();
void handle_button();
void handle_calibration(int ButtonPushed);
void text(BOX_REC Bt,char * A_String);
void drawGrid();

/*****************************************************************/

main (int argc, char *argv[]){

    int d1;
    Window d2;
    XEvent event;
    char temp [10];
    unsigned int Vxh, Vyh;
    unsigned int border;

    Init_Menu_Struct();
    initXwindows (argc, argv);	 /* initialize X stuff */

    AD.Port(2);
 
    if (XGetGeometry(Gdisplay,Gwindow,&d2,&d1,&d1,&Vxh,&Vyh,&border,&Gdepth)==0){
        fprintf (stderr, "Could not get geometry information\n");	
        exit (2);
    }

    Gwidth = Vxh;
    XNextEvent (Gdisplay, &event);
    /* XNextEvent (Gdisplay, &event);		/* get unecessary exposes */
    doEventLoop();	
    /* Resease Ports */
    endXwindows ();	                        /* clear out X stuff */
}

/*****************************************************************/
/* This routine takes all the necessary initialization routines  */
/* and puts them together for clarity.                           */

void initXwindows (int argc, char *argv[]){

    XSizeHints hint;		      /* holds window characteristics */
    char *title = "X Thermomiter";  /* main window name */
    Colormap cmap;
    XColor exact_def;
    int i, j;

    static char* name[]={
        "red",
        "DarkGreen",
	"purple",
	"DarkBlue",
	"wheat1",
	"LightSkyBlue1",
	"Yellow",
	"white",
	"LightBlue",
	"Black",
        "red"
    };

    if (! (Gdisplay = XOpenDisplay (""))){
        fprintf (stderr, "cannot open display\n");
        exit (1);
    }
    Gscreen     = DefaultScreen (Gdisplay);
    hint.x      = 20;
    hint.y      = 20;
    hint.width  = 2 * (BUTTON_WDTH + SP_WDTH) + 2*SP_WDTH;
    hint.height = (BUTTON_HIGT+SP_H_WDTH) + SP_H_WDTH - 5;
    XSize       = hint.width;
    YSize       = hint.height;
    hint.flags  = PPosition | PSize;
    Gforeground = BlackPixel(Gdisplay, Gscreen);
    Gbackground = WhitePixel(Gdisplay, Gscreen);
    Gwindow     = XCreateSimpleWindow (Gdisplay, DefaultRootWindow(Gdisplay),
	                	   hint.x, hint.y, hint.width, hint.height,
		                   0, Gforeground, Gbackground);
    Ggc         = XCreateGC (Gdisplay, Gwindow, 0, 0);
    XSetStandardProperties (Gdisplay, Gwindow, title, title, None, argv, argc, &hint);
    Gevent_mask = StructureNotifyMask | ButtonPressMask | KeyPressMask |
                  ExposureMask | ButtonReleaseMask | ButtonMotionMask | 
                  EnterWindowMask | LeaveWindowMask | PointerMotionHintMask;
    XSelectInput (Gdisplay, Gwindow, Gevent_mask); 
    Gdepth      = DisplayPlanes(Gdisplay, Gscreen);
    cmap        = DefaultColormap (Gdisplay, Gscreen);
    if (Gdepth != 1){		
        for (i = 0; i < MAX_COLORS; i++) {
  	    if (!XParseColor (Gdisplay, cmap, name[i], &exact_def)) {
	        fprintf (stderr, "color %s is not in database. Failed on ->%d\n", name[i],i);
	        exit (1);
	    }
            if (!XAllocColor (Gdisplay, cmap, &exact_def)) {
	        fprintf (stderr, "all color cells are allocated\n");
		exit (1);
	    }
	    color[i] = exact_def.pixel;
        }  /* for */ 
	XSetWindowBackground (Gdisplay, Gwindow, color[BLACK]);
    }  /* if color monitor */
    XSetBackground(Gdisplay, Ggc, Gbackground);
    XSetForeground(Gdisplay, Ggc, Gforeground);
    /* last step for window */
    XMapRaised (Gdisplay, Gwindow);
}

/*******************************************************************/
/* handles user interaction */

void doEventLoop (){

    XEvent event;        /* next X event */
    KeySym key;		 /* dummy for getting keystroke */
    char text[10];	 /* buffer for keystroke */
    int done = FALSE;	 /* quit program flag */
    int start;		 /* start square for knight tour */
    int XLocal,YLocal;
    unsigned int _width, _hight;
    unsigned int border_width;
    int button;
    int Continue;
    int Calibrate;

        event.type = 0;
        Continue = TRUE;
        Calibrate = FALSE;
        while(Continue){
           done = FALSE;
           XWindowEvent (Gdisplay, Gwindow, Gevent_mask, &event);
           if((event.type == EnterNotify) || (event.type == Expose) || (event.type == 22)){
              while (!done){
                if(!Calibrate){ 
                    handle_button();
                    usleep(1000);
	        }
                switch (event.type){
                case LeaveNotify :
                     done = TRUE;
                     break;
                case Expose : 
                     while (XCheckTypedEvent (Gdisplay, Expose, &event));
                     handleExpose();
                     done = TRUE;
        	     break;
                case ButtonPress :
                     if(Calibrate){
                         handle_calibration(event.xbutton.button);
                     }
                     break;
                case MappingNotify :
	       	     XRefreshKeyboardMapping ((XMappingEvent *) &event);
        	     break;
                case KeyPress :
		     if (XLookupString ((XKeyEvent *) &event,text,10,&key,0) == 1 && *text == 'q'){
    	  	        done = TRUE;
                        Continue = FALSE;
		     }
                     if (XLookupString ((XKeyEvent *) &event,text,10,&key,0) == 1 && *text == 't'){
                             AD.Port(2);
		     }
                     if (XLookupString ((XKeyEvent *) &event,text,10,&key,0) == 1 && *text == 'c'){
    	  	        Calibrate = TRUE;
                        handle_calibration(0);
		     }
if (XLookupString ((XKeyEvent *) &event,text,10,&key,0) == 1 && *text == 'e'){
    	  	        Calibrate = FALSE;
		     }
        	     break;
	        } /* switch on type */
                if(!done)
                   XCheckWindowEvent(Gdisplay, Gwindow, Gevent_mask, &event);
             }  /* while not done */
          }  /* End of If statement */
     } /* End of outer for loop */
}  	

/*******************************************************************/
/* handles expose events */

void handleExpose(void){
    XClearWindow (Gdisplay, Gwindow);
    drawGrid ();
    draw_menu();
}

/*******************************************************************/
/* frees up data structures and resources.  */

void endXwindows (void){
    XFreeGC (Gdisplay, Ggc);
    XDestroyWindow (Gdisplay, Gwindow);
    XCloseDisplay (Gdisplay);
}/* endXwindows */

/************************************************************************/
/* Init_Menu_Struct                                                     */
/*                                                                      */
/* This function initializes each structure to be used in drawing       */
/* menu buttons. This structures used are 'Button'. Each 'Button'       */
/* rec is an element in an array which contains all of the buttons      */
/* used by the program. Each Button Rec contains all of the getneral    */
/* information that a button needs to know about.                       */
/*                                                                      */
/*      The structure for a button rec is defined in defs.h and is      */
/*      duplicated in the commented region                              */
/*                                                                      */
/* #define NUM_BUTTONS 12 This is the Total number of active buttons    */
/*                                                                      */
/*            typedef struct box {                                      */
/*                     int Top;                                         */
/*                     int Left;                                        */
/*                     int Bottom;                                      */
/*                     int Right;                                       */
/*                     int OnOff;                                       */
/*                     char * Str;                                      */
/*                     char * Font_Name;                                */
/*             } BOX_REC;                                               */
/*                                                                      */
/* extern BOX_REC Button[NUM_BUTTONS];                                  */
/*                                                                      */
/*                                                                      */


void Init_Menu_Struct() {
         /* First and only Button */
         Button[0].Left       =  BUTTON_WDTH - 9;
         Button[0].Top        =  SP_SHORT_VT;
         Button[0].Right      =  BUTTON_WDTH;
         Button[0].Bottom     =  BUTTON_HIGT;
         Button[0].OnOff      =  OFF;
         Button[0].Bit        =  0;
}

/************************************************************************/
/* Draws menu on the screen                                             */

void draw_menu(){

        int i;
        XFontStruct *font_info;
        unsigned char PortVal;

	XSetForeground (Gdisplay, Ggc, color[LT_BLUE]);
	/* Draw Boxes */
	XDrawRectangle( Gdisplay,Gwindow,Ggc,Button[0].Left,
                                                  Button[0].Top,
                                                  Button[0].Right,
                                                  Button[0].Bottom);
         

	XSetForeground(Gdisplay, Ggc, color[BLUE]);
	XFillRectangle(Gdisplay, Gwindow, Ggc, Button[0].Left+1, 
                                                      Button[0].Top+1, 
                                                      Button[0].Right-1, 
                                                      Button[0].Bottom-1);
	if ((font_info = XLoadQueryFont(Gdisplay,"-adobe-helvetica-medium-r-normal--12-120-75-75-p-67-iso8859-1")) == NULL) {
	  printf("font not loaded\n");
	}
	else {
	  XSetFont(Gdisplay,Ggc,font_info->fid);
	}
	XSetBackground(Gdisplay,Ggc,color[BLUE]);

}

/*******************************************************************/
/* draws strings on screen                                         */

void text(BOX_REC Bt,char * A_String){

     int _X,_Y;

        _X = Bt.Left + 7;
        _Y = (Bt.Bottom/2) + Bt.Top + SP_H_WDTH/2; 

        if(A_String) {
	    XSetForeground(Gdisplay, Ggc,color[WHITE]);
            XDrawImageString(Gdisplay,Gwindow,Ggc,_X,_Y,A_String,strlen(A_String));
        }
        else {
            printf("No String\n"); /* Let user know that there were */
                                  /* no string  names assigned to  */
                                 /* the buttons */
        }
}


/************************************************************************/
/* handles a mouse button press and performs                            */
/* action if selection in menu area occurs                              */

void handle_button(){
        char PutIt[10] = "   ";
        double Voltage;
        double FTemp;
        double CTemp;

	Voltage = AD.Volts();
        CTemp = Temp.VoltsToTemp(Voltage);
        FTemp = Temp.CelToFah(CTemp);
        sprintf(PutIt,"%3.2f  ",FTemp);
        text(Button[0],PutIt);
}

/************************************************************************/
/* handless Calibration of  XVoltmeter                            */

void   handle_calibration(int ButtonPushed){
        char PutIt[10] = "   ";

        if(ButtonPushed == 0){
           sprintf(PutIt,"%2.3f",AD.Ref());
           text(Button[0],PutIt);
        }
        if(ButtonPushed == 1){
           AD.Calibrate(0.01);
           sprintf(PutIt,"%2.3f",AD.Ref());
           text(Button[0],PutIt);
        }
        if(ButtonPushed == 3){
           AD.Calibrate(-0.01);
           sprintf(PutIt,"%2.3f",AD.Ref());
           text(Button[0],PutIt);
        }
}

/****************************************************************************/
/* drawGrid - displays the four colored squares.                            */

void drawGrid (void){

    int i;		/* counter variable */
    int n;		/* offset variable */

    /* Setup upper button menu region */
    XSetForeground(Gdisplay, Ggc, color[YELLOW]);
    XDrawRectangle(Gdisplay, Gwindow, Ggc, 1,0, XSize - 2, YSize - 1);
    XSetForeground(Gdisplay, Ggc, color[DARK_BLUE]);
    XFillRectangle(Gdisplay, Gwindow, Ggc, 2, 1, XSize - 3, YSize - 2);


}  /* drawGrid */
